import pytest
from random import randrange

from c2cgeoform.testing.views import AbstractViewsTests

from getitfixed.models.getitfixed import Category, Event, Issue, Type

from unittest.mock import patch, call, ANY


@pytest.fixture(scope="function")
@pytest.mark.usefixtures("dbsession", "transact")
def issue_test_data(dbsession, transact):
    del transact

    categories = []
    for i in range(5):
        categories.append(
            Category(
                label_en="Category «{}»".format(i),
                label_fr="Catégorie «{}»".format(i),
                email="{}.is@abit.ch".format(i),
            )
        )
    dbsession.add_all(categories)

    types = []
    for i in range(15):
        types.append(
            Type(
                label_en="Type «{}»".format(i),
                label_fr="Type «{}»".format(i),
                category=categories[i % 3],
            )
        )
    dbsession.add_all(types)

    issues = []
    for i in range(0, 10):
        issues.append(
            Issue(
                type=types[i % 15],
                description="{} truite sauvage".format(i),
                localisation="{} rue du pont".format(i),
                firstname="Firstname{}".format(i),
                lastname="Lastname{}".format(i),
                phone="0{} {:02} {:02} {:02} {:02}".format(
                    randrange(1, 10), *[randrange(100) for i in range(4)]
                ),
                email="firstname{0}.lastname{0}@domain.net".format(i),
            )
        )
    dbsession.add_all(issues)

    dbsession.flush()

    yield {"categories": categories, "types": types, "issues": issues}


@pytest.mark.usefixtures("issue_test_data", "test_app")
class TestSemiPrivateIssueViews(AbstractViewsTests):

    _prefix = "/getitfixed/private/issues"

    @patch("getitfixed.views.private.semi_private_events.send_email")
    def test_edit_then_post_comment(
        self, send_email, test_app, issue_test_data, dbsession
    ):
        issue = issue_test_data["issues"][0]
        resp = self.get(test_app, "/{}".format(issue.hash), status=200)

        self._check_mapping(
            resp.html.select("form")[0],
            [
                {"name": "id", "value": str(issue.id), "hidden": True},
                {"name": "type_id", "value": issue.type.label_en, "readonly": True},
                {"name": "description", "value": issue.description, "readonly": True},
                {"name": "description", "value": issue.description, "readonly": True},
                {"name": "localisation", "value": issue.localisation, "readonly": True},
            ],
        )

        form = resp.forms["new_event_form"]
        assert "" == form["id"].value
        assert str(issue.id) == form["issue_id"].value
        assert "" == form["comment"].value

        form["comment"].value = "This is a user comment"

        resp = form.submit("submit", status=302)

        assert (
            "http://localhost/getitfixed/private/issues/{}#existing_events_form".format(
                issue.hash
            )
            == resp.location
        )

        obj = (
            dbsession.query(Event)
            .filter(Event.issue_id == issue.id)
            .order_by(Event.date.desc())
            .first()
        )

        assert "new" == obj.status
        assert "This is a user comment" == obj.comment
        assert "new" == issue.status

        assert send_email.call_count == 1
        assert send_email.mock_calls[0] == call(
            request=ANY,
            to='0.is@abit.ch',
            template_name='update_issue_email',
            template_kwargs={
                'username': 'Firstname0 Lastname0',
                'issue': obj.issue,
                'event': obj,
                'issue-link': 'http://localhost/getitfixed_admin/issues/{}#existing_events_form'.format(obj.issue.hash),
            },
        )

    @patch("getitfixed.emails.email_service.smtplib.SMTP")
    def test_edit_then_post_empty_comment(
        self, smtp_mock, test_app, issue_test_data, dbsession
    ):
        issue = issue_test_data["issues"][0]
        resp = self.get(test_app, "/{}".format(issue.hash), status=200)

        self._check_mapping(
            resp.html.select("form")[0],
            [
                {"name": "id", "value": str(issue.id), "hidden": True},
                {"name": "type_id", "value": issue.type.label_en, "readonly": True},
                {"name": "description", "value": issue.description, "readonly": True},
                {"name": "description", "value": issue.description, "readonly": True},
                {"name": "localisation", "value": issue.localisation, "readonly": True},
            ],
        )
        form = resp.forms["new_event_form"]
        assert "" == form["id"].value
        assert str(issue.id) == form["issue_id"].value
        assert "" == form["comment"].value

        resp = form.submit("submit", status=200)

        assert "http://localhost/getitfixed/events/new" == resp.request.path_url
        assert smtp_mock.call_count == 0



from lingua.extractors import Extractor, Message
from c2c.template.config import config as configuration


class GetItFixedExtractor(Extractor):  # pragma: no cover
    """
    GetItFixed extractor (settings: emails subjects and bodys)
    """

    extensions = [".yaml"]

    def __call__(self, filename, options):
        configuration.init(filename)
        settings = configuration.get_config()

        for path in (
            ("getitfixed", "admin_new_issue_email", "email_subject"),
            ("getitfixed", "admin_new_issue_email", "email_body"),
            ("getitfixed", "new_issue_email", "email_subject"),
            ("getitfixed", "new_issue_email", "email_body"),
            ("getitfixed", "update_issue_email", "email_subject"),
            ("getitfixed", "update_issue_email", "email_body"),
            ("getitfixed", "resolved_issue_email", "email_subject"),
            ("getitfixed", "resolved_issue_email", "email_body"),
        ):
            value = settings
            for key in path:
                value = value[key]
            # yield Message(msgctxt msgid msgid_plural flags comment tcomment location)
            yield Message(None, value, None, [], u"", u"", (filename, "/".join(path)))



# coding=utf-8
from uuid import uuid4
from pkg_resources import resource_filename

from pyramid.i18n import make_localizer

from sqlalchemy import (
    Boolean,
    Column,
    Date,
    DateTime,
    Enum,
    Integer,
    ForeignKey,
    String,
    Text,
)
from sqlalchemy.orm import backref, relationship
from sqlalchemy.sql.expression import func
from sqlalchemy.ext.associationproxy import association_proxy

import geoalchemy2

import colander
from deform.widget import (
    CheckboxWidget,
    FormWidget,
    HiddenWidget,
    SelectWidget,
    SequenceWidget,
    TextAreaWidget,
    TextInputWidget,
)
from c2c.template.config import config

from c2cgeoform import default_map_settings
from c2cgeoform.ext.deform_ext import RelationSelectWidget
from c2cgeoform.ext import colander_ext, deform_ext
from c2cgeoform.models import FileData

from getitfixed.i18n import _
from getitfixed.models.meta import Base
from getitfixed.url import generate_url

schema = "getitfixed"

_getitfixed_config = (config.get_config() or {}).get("getitfixed", {})
_map_config = {**default_map_settings, **_getitfixed_config.get("map", {})}


STATUS_NEW = "new"
STATUS_VALIDATED = "validated"
STATUS_IN_PROGRESS = "in_progress"
STATUS_REPORTER = "waiting_for_reporter"
STATUS_RESOLVED = "resolved"

STATUSES = {
    STATUS_NEW: _("New"),
    STATUS_VALIDATED: _("Validated"),
    STATUS_IN_PROGRESS: _("In progress"),
    STATUS_REPORTER: _("Waiting for Reporter"),
    STATUS_RESOLVED: _("Resolved"),
}

USER_ADMIN = "admin"
USER_REPORTER = "customer"
USER_AUTHORS = {USER_REPORTER: _("Reporter"), USER_ADMIN: _("Administrator")}


class Photo(FileData, Base):
    __tablename__ = "photo"
    __table_args__ = {"schema": schema}
    # Setting unknown to 'preserve' is required in classes used as a
    # FileUpload field.
    __colanderalchemy_config__ = {
        "title": _("Photo"),
        "unknown": "preserve",
        "widget": deform_ext.FileUploadWidget(
            get_url=lambda request, id: request.route_url(
                "c2cgeoform_item", table="photos", id=id
            )
        ),
    }
    issue_id = Column(Integer, ForeignKey("{}.issue.id".format(schema)))


class Category(Base):
    __tablename__ = "category"
    __table_args__ = {"schema": schema}
    __colanderalchemy_config__ = {"title": _("Category"), "plural": _("Categories")}

    id = Column(
        Integer,
        primary_key=True,
        info={"colanderalchemy": {"title": _("Identifier"), "widget": HiddenWidget()}},
    )
    label_fr = Column(
        String(50),
        info={
            "colanderalchemy": {"title": _("Label(fr)"), "widget": TextInputWidget()}
        },
    )
    label_en = Column(
        String(50),
        info={
            "colanderalchemy": {"title": _("Label(en)"), "widget": TextInputWidget()}
        },
    )
    email = Column(
        String(50),
        nullable=False,
        info={"colanderalchemy": {"title": _("Email"), "widget": TextInputWidget()}},
    )

    icon = Column(
        String(150),
        info={"colanderalchemy": {"title": _("Label(en)"), "widget": HiddenWidget()}},
    )


class Type(Base):
    __tablename__ = "type"
    __table_args__ = {"schema": schema}
    __colanderalchemy_config__ = {"title": _("Type"), "plural": _("Types")}

    id = Column(
        Integer,
        primary_key=True,
        info={"colanderalchemy": {"title": _("Identifier"), "widget": HiddenWidget()}},
    )
    label_fr = Column(
        String(50),
        info={
            "colanderalchemy": {"title": _("Label(fr)"), "widget": TextInputWidget()}
        },
    )
    label_en = Column(
        String(50),
        info={
            "colanderalchemy": {"title": _("Label(en)"), "widget": TextInputWidget()}
        },
    )
    category_id = Column(
        Integer,
        ForeignKey("{}.category.id".format(schema)),
        info={
            "colanderalchemy": {
                "title": _("Category"),
                "widget": RelationSelectWidget(
                    Category,
                    "id",
                    "label_fr",
                    order_by="label_fr",
                    default_value=("", _("- Select -")),
                ),
            }
        },
    )
    category = relationship(Category, backref="types")


class Issue(Base):
    __tablename__ = "issue"
    __table_args__ = {"schema": schema}
    __colanderalchemy_config__ = {
        "title": _("Issue"),
        "plural": _("Issues"),
        "widget": FormWidget(fields_template="issue_fields"),
    }

    id = Column(
        Integer,
        primary_key=True,
        info={
            # the `colanderalchemy` property allows to set a custom title for the
            # column or to use a specific widget.
            "colanderalchemy": {"title": _("Identifier"), "widget": HiddenWidget()}
        },
    )
    hash = Column(
        Text,
        nullable=False,
        unique=True,
        default=lambda: str(uuid4()),
        info={"colanderalchemy": {"exclude": True}, "c2cgeoform": {"duplicate": False}},
    )
    request_date = Column(
        Date,
        nullable=False,
        server_default=func.now(),
        info={"colanderalchemy": {"title": _("Request date")}},
    )
    type_id = Column(
        Integer,
        ForeignKey("{}.type.id".format(schema)),
        nullable=False,
        info={
            "colanderalchemy": {
                "title": _("Type"),
                "widget": RelationSelectWidget(
                    Type,
                    "id",
                    "label_en",
                    order_by="label_en",
                    default_value=("", _("- Select -")),
                ),
            }
        },
    )
    type = relationship(Type, info={"colanderalchemy": {"exclude": True}})
    status = Column(
        Enum(*tuple(STATUSES.keys()), native_enum=False, name="status"),
        nullable=False,
        default="new",
        info={
            "colanderalchemy": {
                "title": _("Status"),
                "widget": SelectWidget(
                    values=STATUSES.items(), readonly=True, item_css_class="item-status"
                ),
            }
        },
    )

    @property
    def status_de(self):
        return self.status_i18n("de")

    @property
    def status_en(self):
        return self.status_i18n("en")

    @property
    def status_fr(self):
        return self.status_i18n("fr")

    def status_i18n(self, locale):
        localizer = make_localizer(locale, [resource_filename("getitfixed", "locale")])
        return localizer.translate(STATUSES[self.status])

    description = Column(
        Text,
        nullable=False,
        info={
            "colanderalchemy": {
                "title": _("Description of the problem"),
                "widget": TextAreaWidget(rows=3),
            }
        },
    )
    localisation = Column(
        String(254),
        nullable=False,
        info={"colanderalchemy": {"title": _("Localisation")}},
    )
    geometry = Column(
        geoalchemy2.Geometry("POINT", 4326, management=True),
        info={
            "colanderalchemy": {
                "title": _("Position"),
                "typ": colander_ext.Geometry(
                    "POINT", srid=4326, map_srid=_map_config["srid"]
                ),
                "widget": deform_ext.MapWidget(
                    map_options=_map_config, item_css_class="item-geometry"
                ),
            }
        },
    )
    photos = relationship(
        Photo,
        cascade="all, delete-orphan",
        info={"colanderalchemy": {"title": _("Photos")}},
    )
    firstname = Column(
        String(100), nullable=False, info={"colanderalchemy": {"title": _("Firstname")}}
    )
    lastname = Column(
        String(100), nullable=False, info={"colanderalchemy": {"title": _("Lastname")}}
    )
    phone = Column(
        String(20), nullable=False, info={"colanderalchemy": {"title": _("Phone")}}
    )
    email = Column(
        String(100),
        nullable=False,
        info={
            "colanderalchemy": {
                "title": _("Email"),
                "validator": colander.Email(),
                "description": _(
                    "This field is required to keep you informed about issue events"
                ),
            }
        },
    )
    events = relationship(
        "Event",
        order_by="desc(Event.date)",
        backref=backref("issue", info={"colanderalchemy": {"exclude": True}}),
        info={
            "colanderalchemy": {
                "title": _("Events"),
                "widget": SequenceWidget(readonly=True, item_css_class="item-events"),
            }
        },
    )
    public_events = relationship(
        "Event",
        order_by="desc(Event.date)",
        primaryjoin="and_(Event.issue_id==Issue.id, Event.private==False)",
    )

    category = association_proxy("type", "category")

    def icon_url(self, request):
        icon = (
            self.category.icon
            if self.category and self.category.icon
            else _getitfixed_config.get(
                "default_icon", "static://getitfixed:static/icons/cat-default.png"
            )
        )
        return generate_url(request, icon)


class Event(Base):
    __tablename__ = "event"
    __table_args__ = {"schema": schema}
    __colanderalchemy_config__ = {"title": _("Event"), "plural": _("Events")}
    id = Column(
        Integer,
        primary_key=True,
        info={"colanderalchemy": {"title": _("Identifier"), "widget": HiddenWidget()}},
    )
    issue_id = Column(
        Integer,
        ForeignKey("{}.issue.id".format(schema)),
        nullable=False,
        info={"colanderalchemy": {"title": _("Type"), "widget": HiddenWidget()}},
    )
    status = Column(
        Enum(*tuple(STATUSES.keys()), native_enum=False, name="status"),
        nullable=False,
        info={
            "colanderalchemy": {
                "title": _("Status"),
                "widget": SelectWidget(
                    values=STATUSES.items(), item_css_class="item-status"
                ),
            }
        },
    )
    date = Column(
        DateTime(timezone=True),
        nullable=False,
        server_default=func.now(),
        info={"colanderalchemy": {"title": _("Date"), "widget": HiddenWidget()}},
    )
    comment = Column(
        Text, info={"colanderalchemy": {"title": _("Comment"), "missing": ""}}
    )

    private = Column(
        Boolean,
        info={
            "colanderalchemy": {
                "title": _("Private"),
                "widget": CheckboxWidget(item_css_class="item-private"),
            }
        },
    )
    author = Column(
        Enum(*tuple(USER_AUTHORS.keys()), native_enum=False, name="author"),
        nullable=False,
        default="new",
        info={"colanderalchemy": {"title": _("Author"), "widget": HiddenWidget()}},
    )



from pyramid.httpexceptions import HTTPFound, HTTPNotFound
from pyramid.view import view_config
from pyramid.view import view_defaults
from pyramid.threadlocal import get_current_request
from functools import partial

from sqlalchemy.orm import subqueryload

from colander import SchemaNode, Int
from c2cgeoform.schema import GeoFormSchemaNode
from c2cgeoform.ext.deform_ext import RelationSelectWidget
from c2cgeoform.views.abstract_views import AbstractViews, ListField

from getitfixed.models.getitfixed import Category, Issue, Type, STATUS_NEW
from getitfixed.emails.email_service import send_email

from getitfixed.i18n import _

_list_field = partial(ListField, Issue)

new_schema = GeoFormSchemaNode(Issue, excludes=["request_date", "events", "status"])

new_schema.add_before(
    "type_id",
    SchemaNode(
        Int(),
        name="category_id",
        title=_("Category"),
        widget=RelationSelectWidget(Category, "id", "label_fr", order_by="label_fr"),
    ),
)

follow_schema = GeoFormSchemaNode(
    Issue,
    includes=[
        "status",
        "request_date",
        "type_id",
        "description",
        "localisation",
        "geometry",
    ],
)


def get_types(request):
    return {
        "fields": ["category_id", "type_id"],
        "values": [
            dict(id=o.id, cat=o.category_id, label=o.label_fr)
            for o in request.dbsession.query(Type)
        ],
    }


def get_issue_url(issue):
    # Request isn't available here without breaking changes in c2cgeoform
    # use of get_current_request should be safe here, cf
    # https://docs.pylonsproject.org/projects/pyramid/en/latest/api/threadlocal.html
    request = get_current_request()
    return request.route_url("c2cgeoform_item", id=issue.id)


@view_defaults(match_param=("application=getitfixed", "table=issues"))
class IssueViews(AbstractViews):

    _model = Issue
    _base_schema = new_schema
    _id_field = "id"
    _geometry_field = "geometry"

    _list_fields = [
        _list_field("id", key="url", renderer=get_issue_url),
        _list_field(
            "type", key="category", renderer=lambda issue: issue.type.category.label_fr
        ),
        _list_field("type", renderer=lambda issue: issue.type.label_fr),
        _list_field(
            "type",
            key="icon",
            renderer=lambda issue: partial(
                issue.icon_url, request=get_current_request()
            )(),
        ),
    ]

    MSG_COL = {
        "submit_ok": _(
            "Thank you for your report, "
            "it has been registered with following details, "
            "and will be treated as soon as possible."
        ),
        "copy_ok": _("Please check that the copy fits before submitting."),
    }

    def _base_query(self):
        return (
            super()
            ._base_query()
            .outerjoin(Issue.type)
            .filter(Issue.status.notin_([STATUS_NEW]))
            .options(subqueryload(Issue.type))
        )

    @view_config(
        route_name="c2cgeoform_index",
        renderer="getitfixed:templates/public/issues/index.jinja2",
    )
    def map(self):
        return super().map(self._request.registry.settings["getitfixed"].get("map", {}))

    @view_config(route_name="c2cgeoform_geojson", renderer="json", request_method="GET")
    def geojson(self):
        return super().geojson()

    def _grid_actions(self):
        return []

    def _grid_item_actions(self, item):
        return {"dropdown": []}

    def _item_actions(self, item):
        return []

    @view_config(
        route_name="c2cgeoform_item",
        request_method="GET",
        renderer="getitfixed:templates/public/issues/edit.jinja2",
    )
    def edit(self):
        if self._is_new():
            base_edit = super().edit()
            base_edit["form_render_kwargs"].update({"deps": get_types(self._request)})
            base_edit["item_name"] = _("New")
            return base_edit
        else:
            if not self._request.matchdict["id"].isdigit():
                raise HTTPNotFound()
            base_edit = super().edit(schema=follow_schema, readonly=True)
            base_edit["item_name"] = self._get_object().description
            return base_edit

    # For development/testing purpose
    @view_config(
        route_name="c2cgeoform_item_duplicate",
        request_method="GET",
        renderer="getitfixed:templates/public/issues/edit.jinja2",
    )
    def duplicate(self):
        base_duplicate = super().duplicate()
        base_duplicate["form_render_kwargs"].update({"deps": get_types(self._request)})
        base_duplicate["item_name"] = _("New")
        return base_duplicate

    @view_config(
        route_name="c2cgeoform_item",
        request_method="POST",
        renderer="getitfixed:templates/public/issues/edit.jinja2",
    )
    def save(self):
        base_save = super().save()
        if self._is_new():

            if isinstance(base_save, HTTPFound):
                # Send email to the issue Reporter
                self.send_notification_email(
                    self._obj.email,
                    "new_issue_email",
                    self._request.route_url(
                        "c2cgeoform_item_private",
                        application="getitfixed",
                        id=self._obj.hash,
                    ),
                )
                # Send email to the category Manager
                self.send_notification_email(
                    self._obj.category.email,
                    "admin_new_issue_email",
                    self._request.route_url(
                        "c2cgeoform_item",
                        application="getitfixed_admin",
                        id=self._obj.hash,
                    ),
                )
            else:
                base_save["item_name"] = _("New")
        return base_save

    def send_notification_email(self, send_to, template_name, link):
        send_email(
            request=self._request,
            to=send_to,
            template_name=template_name,
            template_kwargs={
                "username": "{} {}".format(self._obj.firstname, self._obj.lastname),
                "issue": self._obj,
                "issue-link": link,
            },
        )



import pytest
import transaction
import threading
from pyramid import testing
from pyramid.paster import bootstrap
from pyramid.view import view_config
from sqlalchemy.exc import DBAPIError
from webtest import TestApp
from wsgiref.simple_server import make_server

from getitfixed.models import get_engine, get_session_factory, get_tm_session
from getitfixed.scripts import wait_for_db
from getitfixed.scripts.initializedb import init_db


@pytest.fixture(scope="session")
def app_env():
    with bootstrap("tests.ini") as env:
        yield env


@pytest.fixture(scope="session")
@pytest.mark.usefixtures("settings")
def dbsession(settings):
    engine = get_engine(settings)
    wait_for_db(engine)
    init_db(engine, force=True)
    session_factory = get_session_factory(engine)
    session = get_tm_session(session_factory, transaction.manager)
    return session


@pytest.fixture(scope="session")
@pytest.mark.usefixtures("app_env")
def settings(app_env):
    yield app_env.get("registry").settings


@pytest.fixture()  # noqa: F811
@pytest.mark.usefixtures("dbsession", "app_env")
def test_app(request, dbsession, settings, app_env):
    config = testing.setUp(registry=app_env["registry"])
    config.add_request_method(lambda request: dbsession, "dbsession", reify=True)
    """
    config.add_route('user_add', 'user_add')
    config.add_route('users_nb', 'users_nb')
    config.scan(package='acceptance_tests')
    """
    app = config.make_wsgi_app()
    testapp = TestApp(app)
    yield testapp


@pytest.fixture(scope="session")  # noqa: F811
@pytest.mark.usefixtures("dbsession", "app_env")
def selenium_app(request, dbsession, settings, app_env):
    app = app_env.get("app")
    srv = make_server("", 6543, app)
    threading.Thread(target=srv.serve_forever).start()
    yield ()
    srv.shutdown()


@pytest.fixture(scope="function")
@pytest.mark.usefixtures("dbsession")
def transact(dbsession):
    t = dbsession.begin_nested()
    yield
    t.rollback()


def raise_db_error(Table):
    raise DBAPIError("this is a test !", None, None)


@pytest.fixture(scope="function")
@pytest.mark.usefixtures("dbsession")
def raise_db_error_on_query(dbsession):
    query = dbsession.query
    dbsession.query = raise_db_error
    yield
    dbsession.query = query



from c2cgeoform.routes import pregenerator
from getitfixed.models.getitfixed import Event, Issue


def includeme(config):
    config.add_static_view("getitfixed_static", "getitfixed:static", cache_max_age=3600)

    config.add_route("getitfixed", "/getitfixed/")
    config.add_c2cgeoform_application("getitfixed", [("issues", Issue)])

    config.add_route(
        "c2cgeoform_item_private",
        "{application:getitfixed}/private/{table}/{id}",
        pregenerator=pregenerator,
    )

    config.add_route("getitfixed_admin", "/getitfixed_admin")
    config.add_c2cgeoform_application(
        "getitfixed_admin", [("issues", Issue), ("event", Event)]
    )



import pytest

from getitfixed.models.getitfixed import Category, Type, Issue, STATUSES, STATUS_IN_PROGRESS


@pytest.fixture(scope="function")
@pytest.mark.usefixtures("dbsession", "transact")
def issue(dbsession, transact):
    del transact

    category = Category(
        label_en="Category",
        label_fr="Catégorie",
        email="is@abit.ch",
    )

    typ = Type(
        label_en="Type",
        label_fr="Type",
        category=category,
    )

    issue = Issue(
        type=typ,
        description="description",
        localisation="localisation",
        firstname="firstname",
        lastname="lastname",
        phone="0479000000",
        status=STATUS_IN_PROGRESS,
        email="firstname.lastname@domain.net",
    )

    dbsession.add(issue)
    dbsession.flush()

    yield issue


@pytest.mark.usefixtures("issue")
class TestIssue:
    def test_status_i18n(self, issue):
        assert issue.status_de == "In progress"  # FIXME: after de translations are done
        assert issue.status_en == "In progress"
        assert issue.status_fr == "En cours"



from pyramid.view import view_config, view_defaults
from pyramid.httpexceptions import HTTPFound

import colander
from deform.widget import HiddenWidget

from c2cgeoform.schema import GeoFormSchemaNode
from c2cgeoform.views.abstract_views import AbstractViews

from getitfixed.models.getitfixed import Event, USER_REPORTER

from getitfixed.emails.email_service import send_email

base_schema = GeoFormSchemaNode(Event)
base_schema["comment"].missing = colander.required
base_schema["status"].widget = HiddenWidget()
base_schema["private"].widget = HiddenWidget()


@view_defaults(match_param=("application=getitfixed", "table=events"))
class EventViews(AbstractViews):

    _model = Event
    _base_schema = base_schema
    _id_field = "id"

    @view_config(
        route_name="c2cgeoform_item",
        request_method="POST",
        renderer="getitfixed:templates/public/issues/edit.jinja2",
    )
    def save(self):
        resp = super().save()
        if isinstance(resp, HTTPFound):
            if self._obj.author == USER_REPORTER:
                self.send_notification_email(
                    self._obj.issue.category.email,
                    "update_issue_email",
                    self._request.route_url(
                        "c2cgeoform_item",
                        application="getitfixed_admin",
                        table="issues",
                        id=self._obj.issue.hash,
                        _anchor="existing_events_form",
                    ),
                )
            # Redirect to issue form
            return HTTPFound(
                self._request.route_url(
                    "c2cgeoform_item_private",
                    application="getitfixed",
                    table="issues",
                    id=self._obj.issue.hash,
                    _anchor="existing_events_form",
                )
            )

        return resp

    def send_notification_email(self, send_to, template_name, link):
        send_email(
            request=self._request,
            to=send_to,
            template_name=template_name,
            template_kwargs={
                "username": "{} {}".format(
                    self._obj.issue.firstname, self._obj.issue.lastname
                ),
                "issue": self._obj.issue,
                "event": self._obj,
                "issue-link": link,
            },
        )



import os
from typing import Dict, Optional, cast

from plaster_pastedeploy import Loader as BaseLoader


class Loader(BaseLoader):  # type: ignore
    def _get_defaults(
        self, defaults: Optional[Dict[str, str]] = None
    ) -> Dict[str, str]:
        d: Dict[str, str] = {}
        d.update(os.environ)
        d.update(defaults or {})
        settings = super()._get_defaults(d)
        return cast(Dict[str, str], settings)

    def __repr__(self) -> str:
        return 'getitfixed.loader.Loader(uri="{0}")'.format(self.uri)



from mimetypes import guess_type

from pyramid.view import view_config, view_defaults

from c2cgeoform.views.abstract_views import AbstractViews

from getitfixed.models.getitfixed import Photo


@view_defaults(
    match_param=("application=getitfixed_admin", "table=photos"),
    permission="getitfixed_admin",
)
class PhotoViews(AbstractViews):

    _model = Photo
    _id_field = "id"

    @view_config(route_name="c2cgeoform_item", request_method="GET")
    def get(self):
        obj = self._get_object()
        response = self._request.response
        response.body = obj.data
        response.content_type = guess_type(obj.filename)[0]
        return response



from pyramid.view import view_config
from pyramid.httpexceptions import HTTPFound


@view_config(route_name="getitfixed_admin", permission="getitfixed_admin")
def getitfixed(request):
    return HTTPFound(
        request.route_url(
            "c2cgeoform_index", application="getitfixed_admin", table="issues"
        )
    )



from contextlib import suppress
from transaction.interfaces import NoTransaction

# from webtest import TestApp


def setup(env):
    request = env["request"]

    # env['testapp'] = TestApp(env['app'])

    # start a transaction which can be used in the shell
    request.tm.begin()

    # if using the SQLAlchemy backend from our cookiecutter, the dbsession is
    # connected to the transaction manager above
    env["tm"] = request.tm
    env["s"] = request.dbsession
    try:
        yield

    finally:
        with suppress(NoTransaction):
            request.tm.abort()



import time
from sqlalchemy.sql import text


def wait_for_db(connection):
    sleep_time = 1
    max_sleep = 30
    while sleep_time < max_sleep:
        try:
            connection.execute(text("SELECT 1;"))
            return
        except Exception as e:
            print(str(e))
            print("Waiting for the DataBase server to be reachable")
            time.sleep(sleep_time)
            sleep_time *= 2
    exit(1)



from pyramid.i18n import default_locale_negotiator
from pyramid.settings import aslist
from translationstring import TranslationStringFactory


_ = TranslationStringFactory("getitfixed")


def locale_negotiator(request):
    locale_name = default_locale_negotiator(request)

    # available languages are not handled by pyramid itself
    if locale_name is None:
        available = aslist(request.registry.settings["pyramid.available_languages"])
        default = request.registry.settings["pyramid.default_locale_name"]
        locale_name = request.accept_language.lookup(available, default=default)

    return locale_name



from functools import partial

from pyramid.view import view_config, view_defaults
from sqlalchemy.orm import subqueryload

from c2cgeoform.schema import GeoFormSchemaNode
from c2cgeoform.views.abstract_views import ListField

from getitfixed.models.getitfixed import (
    Event,
    Category,
    Issue,
    Type,
    USER_ADMIN,
    STATUS_RESOLVED,
)
from getitfixed.views.private.semi_private_issues import IssueViews

_list_field = partial(ListField, Issue)

route = "c2cgeoform_item"
event_schema = GeoFormSchemaNode(Event)

base_schema = GeoFormSchemaNode(
    Issue,
    includes=[
        "id",
        "status",
        "request_date",
        "type_id",
        "description",
        "localisation",
        "geometry",
        "photos",
        "firstname",
        "lastname",
        "phone",
        "email",
    ],
)

MAX_DESCR_LEN = 40


@view_defaults(
    match_param=("application=getitfixed_admin", "table=issues"),
    permission="getitfixed_admin",
)
class IssueAdminViews(IssueViews):

    _author = USER_ADMIN
    _event_schema = event_schema
    _base_schema = base_schema
    _application = "getitfixed_admin"

    _list_fields = [
        _list_field("id"),
        _list_field("status"),
        _list_field("request_date"),
        _list_field(
            "type_id",
            renderer=lambda issue: issue.type.label_fr,
            sort_column=Type.label_fr,
            filter_column=Type.label_fr,
        ),
        _list_field(
            "description",
            renderer=lambda issue: (issue.description[:MAX_DESCR_LEN] + "…")
            if len(issue.description) > MAX_DESCR_LEN
            else issue.description,
        ),
        _list_field("localisation"),
        _list_field("firstname"),
        _list_field("lastname"),
        _list_field("phone"),
        _list_field("email"),
    ]
    _list_ordered_fields = [Issue.request_date.desc()]

    def _base_query(self):
        query = (
            super()
            ._base_query()
            .outerjoin(Issue.type)
            .options(subqueryload(Issue.type))
        )

        status = self._request.params.get("status", "open")
        if status == "open":
            # return all issues that are not closed
            query = query.filter(Issue.status != STATUS_RESOLVED)

        # filter issues based on category id
        category = int(self._request.params.get("category", 0))
        if category != 0:  # 0 is for all issues
            query = query.filter(Issue.category.has(id=category))
        return query

    @view_config(
        route_name="c2cgeoform_index",
        renderer="getitfixed:templates/admin/issues/index.jinja2",
    )
    def index(self):
        resp = super().index()
        cats = self._request.dbsession.query(Category).all()

        label = (
            "label_fr" if self._request.localizer.locale_name == "fr" else "label_en"
        )
        resp.update(
            {
                "categories": list(
                    map(lambda c: {"id": c.id, "value": getattr(c, label)}, cats)
                )
            }
        )
        return resp

    @view_config(route_name="c2cgeoform_grid", renderer="json")
    def grid(self):
        return super().grid()

    def _grid_actions(self):
        return []

    def _item_actions(self, item):
        return []

    @view_config(
        route_name="c2cgeoform_item",
        request_method="GET",
        renderer="getitfixed:templates/admin/issues/edit.jinja2",
    )
    def edit(self):
        return super().edit()

    @staticmethod
    def events(issue):
        return issue.events

    @view_config(
        route_name="c2cgeoform_item",
        request_method="POST",
        renderer="getitfixed:templates/admin/issues/edit.jinja2",
    )
    def save(self):
        return super().save()



# -*- coding: utf-8 -*-

import logging
import smtplib

from email.mime.text import MIMEText
from email.header import Header

from getitfixed.i18n import _

LOG = logging.getLogger(__name__)


def send_email(request, to, template_name, template_args=[], template_kwargs={}):
    """Send email based on different templates
    Templates are set in the vars.yaml file
    """

    settings = request.registry.settings
    smtp = settings["smtp"]
    if not smtp:
        LOG.warning("Email cannot be sent as smtp service is not configured.")
        return False

    template = settings["getitfixed"][template_name]
    sender = template["email_from"]

    body = _(template["email_body"]).format(template, *template_args, **template_kwargs)

    msg = MIMEText(body, _charset="UTF-8")
    msg["From"] = Header(sender)
    msg["To"] = Header(to)
    # msg['Bcc'] = Header(sender)
    msg["Subject"] = Header(_(template["email_subject"]), "utf-8")

    # Connect to server
    smtp_host = smtp["host"]
    if "ssl" in smtp_host and smtp["ssl"]:
        server = smtplib.SMTP_SSL(smtp_host)
    else:
        server = smtplib.SMTP(smtp_host)
    if "starttls" in smtp and smtp["starttls"]:
        server.starttls()
    if "user" in smtp and smtp["user"]:
        server.login(smtp["user"], smtp["password"])

    # Send message
    server.sendmail(sender, [to, sender], msg.as_string())

    # Diconnect from server
    server.quit()

    return True



import pytest


@pytest.mark.usefixtures("test_app")
class TestI18n:
    def test_index(self, test_app):
        resp = test_app.get("/getitfixed/issues", status=200)

        resp_tmp = resp.click(verbose=True, href="language=en")
        resp_en = resp_tmp.follow()
        html_en = resp_en.html
        news_en = html_en.select('a[href$="/getitfixed/issues/new"]')
        assert 1 == len(news_en)
        assert ["Report an issue"] == list(news_en[0].stripped_strings)

        reference_numbers = html_en.select("th[data-field=id]")
        assert 0 == len(reference_numbers)

        resp_tmp = resp_en.click(verbose=True, href="language=fr")
        resp_fr = resp_tmp.follow()
        html_fr = resp_fr.html
        news_fr = html_fr.select('a[href$="/getitfixed/issues/new"]')
        assert 1 == len(news_fr)
        assert ["Déclarer un problème"] == list(news_fr[0].stripped_strings)



from urllib.parse import urlparse


def generate_url(request, url_definition):
    o = urlparse(url_definition)

    if o.scheme == "static":
        # Definition is a static path ex: static://getitfixed:static/image.png
        return request.static_url("{}{}".format(o.netloc, o.path))

    # Definition is an absolute url ex: https://server.com/static/image.png
    return url_definition



import pytest
import re
import datetime
from random import randrange

from c2cgeoform.testing.views import AbstractViewsTests

from getitfixed.models.getitfixed import (
    Category,
    Event,
    Issue,
    Type,
    STATUS_VALIDATED,
    STATUS_NEW,
)
from getitfixed.views.public.issues import IssueViews

from unittest.mock import patch, call, ANY

STATUSES = [STATUS_VALIDATED, STATUS_NEW]


@pytest.fixture(scope="function")
@pytest.mark.usefixtures("dbsession", "transact")
def issue_test_data(dbsession, transact):
    del transact

    categories = []
    for i in range(5):
        categories.append(
            Category(
                label_en="Category «{}»".format(i),
                label_fr="Catégorie «{}»".format(i),
                email="{}.is@abit.ch".format(i),
                icon="meme{}.gif".format(i),
            )
        )
    dbsession.add_all(categories)

    types = []
    for i in range(15):
        types.append(
            Type(
                label_en="Type «{}»".format(i),
                label_fr="Type «{}»".format(i),
                category=categories[i % 3],
            )
        )
    dbsession.add_all(types)

    issues = []
    for i in range(0, 10):
        issues.append(
            Issue(
                type=types[i % 15],
                description="{} truite sauvage".format(i),
                localisation="{} rue du pont".format(i),
                firstname="Firstname{}".format(i),
                lastname="Lastname{}".format(i),
                phone="0{} {:02} {:02} {:02} {:02}".format(
                    randrange(1, 10), *[randrange(100) for i in range(4)]
                ),
                status=STATUSES[i % 2],
                email="firstname{0}.lastname{0}@domain.net".format(i),
            )
        )
    dbsession.add_all(issues)

    dbsession.flush()

    yield {"categories": categories, "types": types, "issues": issues}


@pytest.mark.usefixtures("issue_test_data", "test_app")
class TestIssueViews(AbstractViewsTests):

    _prefix = "/getitfixed/issues"

    def _obj_by_hash(self, dbsession, hash_):
        return dbsession.query(Issue).filter(Issue.hash == hash_).one_or_none()

    def test_index(self, test_app):
        resp = self.get(test_app, status=200)

    def test_geojson(self, test_app):
        resp = self.get(test_app, "/geojson.json", status=200)
        # We do not show new (not validated) issues
        assert 5 == len(resp.json["features"])

    @patch("getitfixed.views.public.issues.send_email")
    def test_new_then_save(self, send_email, dbsession, test_app, issue_test_data):
        resp = test_app.get("/getitfixed/issues/new", status=200)

        form = resp.form
        assert "" == form["id"].value
        assert "" == form["type_id"].value
        assert "" == form["description"].value
        assert "" == form["localisation"].value
        assert "" == form["firstname"].value
        assert "" == form["lastname"].value
        assert "" == form["phone"].value
        assert "" == form["email"].value

        form["type_id"] = str(issue_test_data["types"][0].id)
        form["description"] = "Description"
        form["localisation"] = "234 long street"
        form["firstname"] = "Andreas"
        form["lastname"] = "Ford"
        form["phone"] = "04 58 48 20 00"
        form["email"] = "andreas.ford@domain.net"

        resp = form.submit("submit", status=302)

        assert (
            IssueViews.MSG_COL["submit_ok"]
            == resp.follow().html.find("div", {"class": "msg-lbl"}).getText()
        )

        id_ = re.match(
            r"http://localhost/getitfixed/issues/(.*)\?msg_col=submit_ok", resp.location
        ).group(1)

        obj = dbsession.query(Issue).get(id_)
        assert datetime.date.today() == obj.request_date
        assert issue_test_data["types"][0] is obj.type
        assert "Description" == obj.description
        assert "234 long street" == obj.localisation
        assert "Andreas" == obj.firstname
        assert "Ford" == obj.lastname
        assert "04 58 48 20 00" == obj.phone
        assert "andreas.ford@domain.net" == obj.email

        assert 2 == send_email.call_count
        assert send_email.mock_calls[0] == call(
            request=ANY,
            to='andreas.ford@domain.net',
            template_name='new_issue_email',
            template_kwargs={
                'username': 'Andreas Ford',
                'issue': obj,
                'issue-link': 'http://localhost/getitfixed/private/issues/{}'.format(obj.hash),
            },
        )
        assert send_email.mock_calls[1] == call(
            request=ANY,
            to='0.is@abit.ch',
            template_name='admin_new_issue_email',
            template_kwargs={
                'username': 'Andreas Ford',
                'issue': obj,
                'issue-link': 'http://localhost/getitfixed_admin/issues/{}'.format(obj.hash),
            },
        )

    def test_open(self, dbsession, test_app, issue_test_data):

        issue = dbsession.query(Issue).first()
        resp = test_app.get("/getitfixed/issues/{}".format(issue.id), status=200)
        self._check_mapping(
            resp.html.select("form")[0],
            [
                {"name": "description", "value": issue.description, "readonly": True},
                {"name": "localisation", "value": issue.localisation, "readonly": True},
            ],
        )



import pytest
from pyramid import testing
from pyramid.paster import bootstrap

from getitfixed.url import generate_url


@pytest.mark.usefixtures("app_env")
def test_generate_url(app_env):
    request = app_env["request"]
    assert "https://server.com/static/image.png" == generate_url(request, "https://server.com/static/image.png")
    assert "/static/image.png" == generate_url(request, "/static/image.png")
    assert "static/image.png" == generate_url(request, "static/image.png")
    # Static url with explicit package name
    assert "http://localhost/getitfixed_static/image.png" == generate_url(request, "static://getitfixed:static/image.png")
    # Default to registry.package_name
    assert "http://localhost/getitfixed_static/image.png" == generate_url(request, "static://static/image.png")

def test_gmf_2_5():
    with bootstrap("tests.ini") as env:
        # Compatibility with GMF 2.5
        config = testing.setUp(registry=env["registry"])
        config.add_static_view(name="static", path="/etc/geomapfish/static")
        request = env["request"]
        assert "http://localhost/static/image.png" == generate_url(request, "static:///etc/geomapfish/static/image.png")



import os
from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(here, "README.rst")) as f:
    README = f.read()
with open(os.path.join(here, "CHANGES.txt")) as f:
    CHANGES = f.read()
with open(os.path.join(here, "requirements.txt")) as f:
    REQUIRES = f.read()

setup(
    name="getitfixed",
    version=os.environ.get("VERSION", "1.0.14"),
    description="getitfixed",
    long_description=README + "\n\n" + CHANGES,
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Framework :: Pyramid",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: WSGI :: Application",
    ],
    author="",
    author_email="",
    url="",
    keywords="web wsgi bfg pylons pyramid",
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    test_suite="getitfixed",
    install_requires=REQUIRES,
    entry_points="""\
        [paste.app_factory]
            main = getitfixed:main
        [console_scripts]
            initialize_getitfixed_db = getitfixed.scripts.initializedb:main
        [lingua.extractors]
            getitfixed = getitfixed.lingua_extractor:GetItFixedExtractor
        [plaster.loader_factory]
            getitfixed = getitfixed.loader:Loader
            getitfixed+ini = getitfixed.loader:Loader
            getitfixed+egg = getitfixed.loader:Loader
        [plaster.wsgi_loader_factory]
            getitfixed = getitfixed.loader:Loader
            getitfixed+ini = getitfixed.loader:Loader
            getitfixed+egg = getitfixed.loader:Loader
    """,
)



from pyramid.view import view_config
from pyramid.httpexceptions import HTTPFound


@view_config(route_name="getitfixed")
def getitfixed(request):
    return HTTPFound(
        request.route_url("c2cgeoform_index", application="getitfixed", table="issues")
    )



from functools import partial

from pyramid.view import view_config, view_defaults
from pyramid.httpexceptions import HTTPNotFound
from deform import Button, Form
from deform.widget import HiddenWidget

from c2cgeoform.schema import GeoFormSchemaNode
from c2cgeoform.views.abstract_views import AbstractViews, ListField

from getitfixed.models.getitfixed import USER_ADMIN, USER_REPORTER, Event, Issue

from getitfixed.i18n import _


_list_field = partial(ListField, Issue)

base_schema = GeoFormSchemaNode(
    Issue,
    excludes=["email", "firstname", "lastname", "phone", "events", "public_events"],
)
event_schema = GeoFormSchemaNode(Event)
event_schema["status"].widget = HiddenWidget()
event_schema["private"].widget = HiddenWidget()


@view_defaults(match_param="table=issues")
class IssueViews(AbstractViews):
    _model = Issue
    _base_schema = base_schema
    _id_field = "hash"
    _author = USER_REPORTER
    _event_schema = event_schema
    _application = "getitfixed"

    @view_config(
        route_name="c2cgeoform_item_private",
        request_method="GET",
        renderer="getitfixed:templates/admin/issues/edit.jinja2",
    )
    def edit(self):
        if self._is_new():
            return HTTPNotFound()
        else:
            # Create a readonly issue form
            resp = super().edit(readonly=True)

            issue = self._get_object()

            event = Event(issue_id=issue.id)
            event.status = issue.status
            event.author = self._author
            event_form = Form(
                self._event_schema,
                formid="new_event_form",
                buttons=[Button(name="formsubmit", title=_("Submit"))],
                action=self._request.route_url(
                    "c2cgeoform_item",
                    application=self._application,
                    table="events",
                    id="new",
                ),
            )
            resp.update(
                {
                    "event_form": event_form,
                    "event_form_render_args": (event_form.schema.dictify(event),),
                    "event_form_render_kwargs": {
                        "request": self._request,
                        "user_admin": USER_ADMIN,
                        "obj": issue,
                    },
                }
            )
            resp.update({"events": self.events(issue)})
            return resp

    @staticmethod
    def events(issue):
        return issue.public_events



from pkg_resources import resource_filename
from pyramid.config import Configurator

from c2c.template.config import config as configuration
import c2cgeoform

search_paths = (
    resource_filename("getitfixed", "templates/widgets"),
) + c2cgeoform.default_search_paths
c2cgeoform.default_search_paths = search_paths


def main(global_config, **settings):
    """ This function returns a Pyramid WSGI application.
    """
    # Update the settings object from the YAML application config file
    configuration.init(settings.get("app.cfg"))
    settings.update(configuration.get_config())

    config = Configurator(
        settings=settings, locale_negotiator="getitfixed.i18n.locale_negotiator"
    )

    config.include("getitfixed")
    config.include("getitfixed.models")
    c2cgeoform.routes.register_routes(config)

    def notfound_view(request):
        request.response.status = 404
        return {}

    config.add_notfound_view(
        notfound_view, append_slash=True, renderer="getitfixed:templates/404.jinja2"
    )

    return config.make_wsgi_app()


def includeme(config: Configurator):
    config.include("pyramid_jinja2")
    config.include("c2cgeoform")
    config.include(".routes")
    config.add_translation_dirs("getitfixed:locale")
    config.scan()



# coding=utf-8
import os
import sys
import transaction
import random
from datetime import date, timedelta
from random import randrange

from pyramid.scripts.common import parse_vars, get_config_loader

from getitfixed.scripts import wait_for_db

ICONS = ["gif-black.png", "gif-green.png", "gif-red.png"]


def usage(argv):
    cmd = os.path.basename(argv[0])
    print(
        "usage: %s <config_uri> [var=value]\n"
        '(example: "%s development.ini")' % (cmd, cmd)
    )
    sys.exit(1)


def main(argv=sys.argv):
    if len(argv) < 2:
        usage(argv)
    config_uri = argv[1]
    options = parse_vars(argv[2:])

    loader = get_config_loader(config_uri)
    loader.setup_logging()
    settings = loader.get_wsgi_app_settings(defaults=options)

    # Import the model after settings are loaded
    from getitfixed.models import get_engine

    engine = get_engine(settings)
    wait_for_db(engine)

    with engine.begin() as connection:
        init_db(
            connection, force="--force" in options, with_data="--with-data" in options
        )


def init_db(connection, force=False, with_data=False):
    # Import the model after settings are loaded
    from getitfixed.models import get_session_factory, get_tm_session
    from getitfixed.models.getitfixed import schema
    from getitfixed.models.meta import Base

    if force:
        if schema_exists(connection, schema):
            connection.execute("DROP SCHEMA {} CASCADE;".format(schema))

    if not schema_exists(connection, schema):
        connection.execute('CREATE SCHEMA "{}";'.format(schema))

    Base.metadata.create_all(connection)

    session_factory = get_session_factory(connection)

    with transaction.manager:
        dbsession = get_tm_session(session_factory, transaction.manager)
        if with_data:
            setup_test_data(dbsession)


def schema_exists(connection, schema_name):
    sql = """
SELECT count(*) AS count
FROM information_schema.schemata
WHERE schema_name = '{}';
""".format(
        schema_name
    )
    result = connection.execute(sql)
    row = result.first()
    return row[0] == 1


def get_geometry(dbsession):
    coord_x = random.uniform(5.9559113, 10.4922941)
    coord_y = random.uniform(45.817995, 47.8084648)
    result_proxy = dbsession.execute(
        "SELECT ST_SetSRID( ST_Point( {}, {}), 4326) as geom;".format(coord_x, coord_y)
    )
    return result_proxy.first()[0]


def setup_test_data(dbsession):
    # Import the model after settings are loaded
    from getitfixed.models.getitfixed import Issue, Category, Type

    if dbsession.query(Category).count() == 0:
        for i in range(4):
            dbsession.add(
                Category(
                    label_en="Category «{}»".format(i),
                    label_fr="Catégorie «{}»".format(i),
                    email="test{}@toto.com".format(i),
                    icon="static://getitfixed:static/icons/{}".format(ICONS[i % 3])
                    if i != 3
                    else None,
                )
            )
    if dbsession.query(Type).count() == 0:
        for i in range(15):
            dbsession.add(
                Type(
                    label_en="Type «{}»".format(i),
                    label_fr="Type «{}»".format(i),
                    category_id=(i % 4) + 1,
                )
            )
    if dbsession.query(Issue).count() == 0:
        for i in range(100):
            dbsession.add(_issue(i, (i % 15) + 1, dbsession))


DESCRIPTIONS = ("Déchets sur la voie publique", "Nid de poule")

FIRSTNAMES = ("Dale", "Teresa", "Beatrice", "Darcie")

LASTNAMES = ("Lamb", "Evans", "Alexander", "Rowe", "Ford")


def get_value(col, i):
    return col[i % len(col)]


def _issue(i, type_id, dbsession):
    from getitfixed.models.getitfixed import Issue, STATUSES

    STATUSES = list(STATUSES.keys())

    issue = Issue(
        request_date=date.today() - timedelta(days=100 - i),
        type_id=type_id,
        description=get_value(DESCRIPTIONS, i),
        localisation="{} rue du pont".format(i),
        geometry=get_geometry(dbsession),
        # position
        # photos
        firstname=get_value(FIRSTNAMES, i),
        lastname=get_value(LASTNAMES, i),
        phone="0{} {:02} {:02} {:02} {:02}".format(
            randrange(1, 10), *[randrange(100) for i in range(4)]
        ),
        status=STATUSES[i % 5],
    )
    issue.email = "{}.{}@domain.net".format(
        issue.firstname.lower(), issue.lastname.lower()
    )
    return issue
